@@ -8,7 +8,7 @@ use Regexp::RegGrp;
 
 # =========================================================================== #
 
-our $VERSION = '1.006003';
+our $VERSION = "2.01";
 
 our @BOOLEAN_ACCESSORS = ( 'no_compress_comment', 'remove_copyright' );
 
@@ -215,6 +215,14 @@ sub compress {
     return $self->{_compress};
 }
 
+# these variables are used in the closures defined in the init function
+# below - we have to use globals as using $self within the closures leads
+# to a reference cycle and thus memory leak, and we can't scope them to
+# the init method as they may change. they are set by the minify sub
+our $reggrp_comments;
+our $reggrp_clean;
+our $reggrp_whitespace;
+
 sub init {
     my $class = shift;
     my $self  = {};
@@ -293,11 +301,11 @@ sub init {
     $self->{comments}->{reggrp_data}->[-2]->{replacement} = sub {
         my $submatches = $_[0]->{submatches};
         if ( $submatches->[0] eq '@' ) {
-            $self->reggrp_comments()->exec( \$submatches->[1] );
-            $self->reggrp_clean()->exec( \$submatches->[1] );
-            $self->reggrp_whitespace()->exec( \$submatches->[1] );
+            $reggrp_comments->exec( \$submatches->[1] );
+            $reggrp_clean->exec( \$submatches->[1] );
+            $reggrp_whitespace->exec( \$submatches->[1] );
 
-            return sprintf( "//%s%s\n%s", @{$submatches} );
+            return sprintf( "//%s%s\n%s", @{$submatches}[0 .. 2] );
         }
         return sprintf( "\n%s", $submatches->[2] );
     };
@@ -307,9 +315,9 @@ sub init {
         if ( $submatches->[0] =~ /^\/\*\@(.*)\@\*\/$/sm ) {
             my $cmnt = $1;
 
-            $self->reggrp_comments()->exec( \$cmnt );
-            $self->reggrp_clean()->exec( \$cmnt );
-            $self->reggrp_whitespace()->exec( \$cmnt );
+            $reggrp_comments->exec( \$cmnt );
+            $reggrp_clean->exec( \$cmnt );
+            $reggrp_whitespace->exec( \$cmnt );
 
             return sprintf( '/*@%s@*/ %s', $cmnt, $submatches->[1] );
         }
@@ -373,6 +381,11 @@ sub minify {
         }
     }
 
+	# (re)initialize variables used in the closures
+	$reggrp_comments = $self->reggrp_comments;
+	$reggrp_clean = $self->reggrp_clean;
+	$reggrp_whitespace = $self->reggrp_whitespace;
+
     my $copyright_comment = '';
 
     if ( ${$javascript} =~ /$COPYRIGHT_COMMENT/ism ) {
@@ -413,7 +426,7 @@ sub minify {
         my %shrunk_vars = map { $_ => 1 } ( ${$javascript} =~ /$SHRINK_VARS->{SHRUNK}/g );
 
         my $cnt = 0;
-        foreach my $shrunk_var ( keys( %shrunk_vars ) ) {
+        foreach my $shrunk_var ( sort keys( %shrunk_vars ) ) {
             my $short_id;
             do {
                 $short_id = $self->_encode52( $cnt++ );
@@ -438,7 +451,7 @@ sub minify {
             $words->{$_}->{count}++;
         }
 
-        WORD: foreach my $word ( sort { $words->{$b}->{count} <=> $words->{$a}->{count} } keys( %{$words} ) ) {
+        WORD: foreach my $word ( sort { $words->{$b}->{count} <=> $words->{$a}->{count} } sort keys( %{$words} ) ) {
 
             if ( exists( $words->{$word}->{encoded} ) and $words->{$word}->{encoded} eq $word ) {
                 next WORD;
@@ -473,7 +486,7 @@ sub minify {
 
         my ( @pk, @pattern ) = ( (), () );
 
-        foreach ( sort { $words->{$a}->{index} <=> $words->{$b}->{index} } keys( %{$words} ) ) {
+        foreach ( sort { $words->{$a}->{index} <=> $words->{$b}->{index} } sort keys( %{$words} ) ) {
             $packed_length -= ( $words->{$_}->{count} * $words->{$_}->{minus} );
 
             if ( $words->{$_}->{encoded} ne $_ ) {
@@ -645,7 +658,7 @@ sub _store_block_data {
         if ( $do_shrink ) {
 
             my $cnt = 0;
-            foreach my $block_var ( keys( %block_vars ) ) {
+            foreach my $block_var ( sort keys( %block_vars ) ) {
                 if ( length( $block_var ) ) {
                     while ( $block =~ /$SHRINK_VARS->{PREFIX}\Q$cnt\E\b/ ) {
                         $cnt++;
@@ -712,9 +725,13 @@ __END__
 
 JavaScript::Packer - Perl version of Dean Edwards' Packer.js
 
+=for html
+<a href='https://travis-ci.org/leejo/javascript-packer-perl?branch=master'><img src='https://travis-ci.org/leejo/javascript-packer-perl.svg?branch=master' alt='Build Status' /></a>
+<a href='https://coveralls.io/r/leejo/javascript-packer-perl'><img src='https://coveralls.io/repos/leejo/javascript-packer-perl/badge.png?branch=master' alt='Coverage Status' /></a>
+
 =head1 VERSION
 
-Version 1.006003
+Version 2.01
 
 =head1 DESCRIPTION
 
@@ -833,12 +850,13 @@ A scalar is requested by the context. The input will remain unchanged.
 
 =head1 AUTHOR
 
-Merten Falk, C<< <nevesenin at cpan.org> >>
+Merten Falk, C<< <nevesenin at cpan.org> >>. Now maintained by Lee
+Johnson (LEEJO)
 
 =head1 BUGS
 
 Please report any bugs or feature requests through the web interface at
-L<http://github.com/nevesenin/javascript-packer-perl/issues>.
+L<http://github.com/leejo/javascript-packer-perl/issues>.
 
 =head1 SUPPORT
 
